/*=====================================================================
File: AddInEntryPoint.cs

Summary: 

Date : Fev 2007

Origin: Open Cellar ! Project

Revision:

Aut: Matthieu DUCROCQ

---------------------------------------------------------------------
Copyright (C) N/A
=====================================================================*/
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Collections;
using OpenCellar.Application;
using OpenCellar.Framework;
using OpenCellar.Controls;
using OpenCellar.Controls.Interface;
using OpenCellar.Controls.Manager;

namespace OpenCellar.AddIn.AlternateColorGrid
{
	/// <summary>
	/// AddInEntryPoint.
	/// </summary>
	public class AddInEntryPoint : IAddIn, IHotKey
	{
		const string TOOL_TIP = "Complment alternance de couleur";

		private IApplication m_application = null;

		/// <summary>
		/// Initialise une instance de classe AddInEntryPoint.
		/// </summary>
		public AddInEntryPoint(){}

		public void OnUnInitialize()
		{
			m_application.NewWindow -= new WindowEventHandler(OnNewWindow);
			m_application.HotKeys.UnRegister(this);
		}

		public void OnCommand(int command){}

		public void OnRegisterMenus(AddInMenus addInMenus){}

		public void OnInitialize(IApplication application)
		{
			m_application = application;
			m_application.HotKeys.Register(this);
			m_application.NewWindow += new WindowEventHandler(OnNewWindow);
		}

		private void OnNewWindow(object sender, WindowEventArgs e)
		{
			//
			// Si la fenetre est de type grille des vins
			//
			if(e.Window.Type == WindowType.WineList)
			{
				SkinWindow(e.Window);
			}
		}

		private void SkinWindow(IWindow window)
		{
			const string ID_BTN = "ALTERNATE_GRID";

			ICommandBarItem button = new ButtonBarItem(ID_BTN);

			window.CommandBars.GetBar(0).AddItem(button); 

			FindAndPatch(button);

			window.CommandBars.GetBar(0).RemoveItem(ID_BTN);
		}

		/// <summary>
		/// Cherche apres la grille et applique
		/// la logique de prsentation.
		/// </summary>
		/// <param name="item"></param>
		private void FindAndPatch(ICommandBarItem item)
		{
			Form parent = item.GetControl().FindForm();

			//
			// Ne devrait pas arriver...
			//
			if(parent == null)
			{
				return;
			}

			bool flag = false;
			
			//
			// La grille est situe dans un controle de niveau 2
			//
			for (int i = 0; i < parent.Controls.Count; i++)
			{
				for (int j = 0; j < parent.Controls[i].Controls.Count; j++)
				{
					if(parent.Controls[i].Controls[j] is DataGrid)
					{
						((DataGrid)parent.Controls[i].Controls[j]).TableStyles[0].BackColor 
							= GetColorSettings("BackColor", DefaultBackColor);

						((DataGrid)parent.Controls[i].Controls[j]).TableStyles[0].AlternatingBackColor 
							= GetColorSettings("AlternativeBackColor", DefaultAlternativeBackColor);

						flag = true;
						break;
					}
				}

				if(flag)
				{
					break;
				}
			}
		}

		public bool HandleKey(HotKeyMessage msg)
		{
			if((m_application.ActiveCellar != null)
				&& (m_application.ActiveWindow != null)
				&& (m_application.ActiveWindow is IWineListWindow)
				&& (msg.Shift)
				&& (msg.KeyCode == Keys.F9))
			{
				ShowSettingsPage();
			}
			return false;
		}

		/// <summary>
		/// Affiche la page de configuration.
		/// </summary>
		private void ShowSettingsPage()
		{
			using(PrefColorsPage pcp = new PrefColorsPage())
			{
				pcp.Color1 = GetColorSettings("BackColor", DefaultBackColor);
				pcp.Color2 = GetColorSettings("AlternativeBackColor", DefaultAlternativeBackColor);
				if(pcp.ShowDialog() == DialogResult.OK)
				{
					SetColorPref("BackColor", pcp.Color1);
					SetColorPref("AlternativeBackColor", pcp.Color2);

					//
					// Applique les changements.
					//
					SkinWindow(m_application.ActiveWindow);
				}
			}
		}

		/// <summary>
		/// Valeur par dfaut ligne 1.
		/// </summary>
		static Color DefaultBackColor = Color.White;

		/// <summary>
		/// Valeur par dfaut ligne 2.
		/// </summary>
		static Color DefaultAlternativeBackColor = Color.WhiteSmoke;

		/// <summary>
		/// Enregistre la couleur selectionne par l'utilisateur.
		/// </summary>
		/// <param name="name"></param>
		/// <param name="color"></param>
		private void SetColorPref(string name,
			Color color)
		{
			m_application.AppSettings["__AddInColor"][name] = color.ToArgb().ToString();
		}

		/// <summary>
		/// Obtient la couleur enregistre dans les prfrences.
		/// </summary>
		/// <param name="name"></param>
		/// <param name="defaultColor"></param>
		/// <returns></returns>
		private Color GetColorSettings(string name, Color defaultColor)
		{
			int colorValue = -1;

			try
			{
				colorValue = Convert.ToInt32(m_application.AppSettings["__AddInColor"][name]);
			}
			catch
			{
				//
				// ZZzzZZ
				//
			}

			if(colorValue != -1)
			{
				return(Color.FromArgb(colorValue));
			}

			return(defaultColor);
		}
	}
}
